@@ -1,10 +1,14 @@
-__version__ = "0.1.4"
+__version__ = "0.2.8"
 
 from .client_config import ClientNotFoundError
 from .components.buildx.cli_wrapper import Builder
 from .components.config.cli_wrapper import Config
 from .components.container.cli_wrapper import Container, ContainerStats
-from .components.context.cli_wrapper import Context
+from .components.context.cli_wrapper import (
+    Context,
+    DockerContextConfig,
+    KubernetesContextConfig,
+)
 from .components.image.cli_wrapper import Image
 from .components.network.cli_wrapper import Network
 from .components.node.cli_wrapper import Node
@@ -20,3 +24,26 @@ from .exceptions import DockerException
 
 # alias
 docker = DockerClient()
+
+__all__ = [
+    "Builder",
+    "ClientNotFoundError",
+    "Config",
+    "Container",
+    "ContainerStats",
+    "Context",
+    "DockerContextConfig",
+    "DockerException",
+    "Image",
+    "KubernetesContextConfig",
+    "Network",
+    "Node",
+    "Plugin",
+    "Secret",
+    "Service",
+    "Stack",
+    "SystemInfo",
+    "Task",
+    "Volume",
+    "DockerClient"
+]
