@@ -2,12 +2,11 @@ from __future__ import annotations
 
 import inspect
 import json
+import re
 from datetime import datetime, timedelta
 from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, overload
 
-import pydantic
-
 from ...client_config import ClientConfig, DockerCLICaller, ReloadableObjectFromJson
 from ...components.image import cli_wrapper as image_cli_wrapper
 from ...components.network import cli_wrapper as network_cli_wrapper
@@ -53,7 +52,7 @@ class Container(ReloadableObjectFromJson):
         return run(self.docker_cmd + ["container", "inspect", reference])
 
     def _parse_json_object(self, json_object: Dict[str, Any]):
-        return ContainerInspectResult.parse_obj(json_object)
+        return ContainerInspectResult.model_validate(json_object)
 
     def _get_inspect_result(self) -> ContainerInspectResult:
         """Only there to allow tools to know the return type"""
@@ -171,6 +170,21 @@ class Container(ReloadableObjectFromJson):
     # --------------------------------------------------------------------
     # public methods
 
+    def attach(
+        self,
+        detach_keys: Optional[str] = None,
+        stdin: bool = True,
+        sig_proxy: bool = True,
+    ) -> None:
+        """Attach local standard input, output, and error streams to a running container.
+
+        Alias: `docker.attach(...)`
+
+        See the [`docker.container.attach`](../sub-commands/container.md#attach) command for
+        information about the arguments.
+        """
+        ContainerCLI(self.client_config).attach(self, detach_keys, not stdin, sig_proxy)
+
     def commit(
         self,
         tag: Optional[str] = None,
@@ -258,14 +272,16 @@ class Container(ReloadableObjectFromJson):
         tail: Optional[int] = None,
         timestamps: bool = False,
         until: Union[None, datetime, timedelta] = None,
-    ) -> str:
+        follow: bool = False,
+        stream: bool = False,
+    ) -> Union[str, Iterable[Tuple[str, bytes]]]:
         """Returns the logs of the container
 
         See the [`docker.container.logs`](../sub-commands/container.md#logs) command for
         information about the arguments.
         """
         return ContainerCLI(self.client_config).logs(
-            self, details, since, tail, timestamps, until
+            self, details, since, tail, timestamps, until, follow, stream
         )
 
     def pause(self) -> None:
@@ -349,9 +365,35 @@ class ContainerCLI(DockerCLICaller):
         super().__init__(*args, **kwargs)
         self.remove = self.remove
 
-    def attach(self):
-        """Not yet implemented"""
-        raise NotImplementedError
+    def attach(
+        self,
+        container: ValidContainer,
+        detach_keys: Optional[str] = None,
+        stdin: bool = True,
+        sig_proxy: bool = True,
+    ) -> None:
+        """Attach local standard input, output, and error streams to a running container
+
+        Alias: `docker.attach(...)`
+
+        # Arguments
+            container: The running container to attach to
+            detach_keys: Override the key sequence for detaching a container
+            stdin: Attach STDIN
+            sig_proxy: Proxy all received signals to the process (default true)
+
+        # Raises
+            `dockertown.exceptions.NoSuchContainer` if the container does not exists.
+        """
+        self.inspect(container)
+
+        full_cmd = self.docker_cmd + ["attach"]
+        full_cmd.add_simple_arg("--detach-keys", detach_keys)
+        full_cmd.add_flag("--no-stdin", not stdin)
+        full_cmd.add_flag("--sig-proxy", sig_proxy)
+        full_cmd.append(container)
+
+        run(full_cmd, tty=True)
 
     def commit(
         self,
@@ -447,6 +489,7 @@ class ContainerCLI(DockerCLICaller):
         cap_add: List[str] = [],
         cap_drop: List[str] = [],
         cgroup_parent: Optional[str] = None,
+        cgroupns: Optional[str] = None,
         cidfile: Optional[ValidPath] = None,
         cpu_period: Optional[int] = None,
         cpu_quota: Optional[int] = None,
@@ -510,6 +553,7 @@ class ContainerCLI(DockerCLICaller):
         privileged: bool = False,
         publish: List[ValidPortMapping] = [],
         publish_all: bool = False,
+        pull: str = "missing",
         read_only: bool = False,
         restart: Optional[str] = None,
         remove: bool = False,
@@ -530,6 +574,7 @@ class ContainerCLI(DockerCLICaller):
         volume_driver: Optional[str] = None,
         volumes_from: List[ValidContainer] = [],
         workdir: Optional[ValidPath] = None,
+        x_passthrough_args: List[str] = [],
     ) -> Container:
         """Creates a container, but does not start it.
 
@@ -546,7 +591,13 @@ class ContainerCLI(DockerCLICaller):
 
         The arguments are the same as [`docker.run`](#run).
         """
-        image_cli_wrapper.ImageCLI(self.client_config)._pull_if_necessary(image)
+
+        image_cli = image_cli_wrapper.ImageCLI(self.client_config)
+        if pull == "missing":
+            image_cli._pull_if_necessary(image)
+        elif pull == "always":
+            image_cli.pull(image)
+
         full_cmd = self.docker_cmd + ["create"]
 
         add_hosts = [f"{host}:{ip}" for host, ip in add_hosts]
@@ -559,6 +610,7 @@ class ContainerCLI(DockerCLICaller):
         full_cmd.add_args_list("--cap-drop", cap_drop)
 
         full_cmd.add_simple_arg("--cgroup-parent", cgroup_parent)
+        full_cmd.add_simple_arg("--cgroupns", cgroupns)
         full_cmd.add_simple_arg("--cidfile", cidfile)
 
         full_cmd.add_simple_arg("--cpu-period", cpu_period)
@@ -653,6 +705,9 @@ class ContainerCLI(DockerCLICaller):
         self._add_publish_to_command(full_cmd, publish)
         full_cmd.add_flag("--publish-all", publish_all)
 
+        if pull == "never":
+            full_cmd.add_simple_arg("--pull", "never")
+
         full_cmd.add_flag("--read-only", read_only)
         full_cmd.add_simple_arg("--restart", restart)
         full_cmd.add_flag("--rm", remove)
@@ -684,6 +739,8 @@ class ContainerCLI(DockerCLICaller):
 
         full_cmd.add_simple_arg("--workdir", workdir)
 
+        full_cmd += x_passthrough_args
+
         full_cmd.append(image)
         full_cmd += command
         return Container(self.client_config, run(full_cmd), is_immutable_id=True)
@@ -767,6 +824,23 @@ class ContainerCLI(DockerCLICaller):
         # Raises
             `dockertown.exceptions.NoSuchContainer` if the container does not exists.
         """
+        if not isinstance(command, list):
+            error_message = (
+                "When calling docker.execute(), the second argument ('command') "
+                "should be a list. "
+                "Here are some examples:"
+                "docker.execute('somecontainer', ['ls']), "
+                "docker.execute('somecontainer', ['cat', '/some/file.txt'])"
+            )
+            if isinstance(command, str):
+                # boy this is the most common error in the world
+                if isinstance(container, str):
+                    container = self.inspect(container)
+                error_message += (
+                    f" In your case, command should not be a string. "
+                    f"You can try docker.execute('{container.name}', {command.split()}, ...)."
+                )
+            raise TypeError(error_message)
         full_cmd = self.docker_cmd + ["exec"]
 
         full_cmd.add_flag("--detach", detach)
@@ -872,7 +946,7 @@ class ContainerCLI(DockerCLICaller):
     def kill(
         self,
         containers: Union[ValidContainer, List[ValidContainer]],
-        signal: str = None,
+        signal: Optional[str] = None,
     ) -> None:
         """Kill a container.
 
@@ -1119,6 +1193,7 @@ class ContainerCLI(DockerCLICaller):
         cap_add: List[str] = [],
         cap_drop: List[str] = [],
         cgroup_parent: Optional[str] = None,
+        cgroupns: Optional[str] = None,
         cidfile: Optional[ValidPath] = None,
         cpu_period: Optional[int] = None,
         cpu_quota: Optional[int] = None,
@@ -1183,6 +1258,7 @@ class ContainerCLI(DockerCLICaller):
         privileged: bool = False,
         publish: List[ValidPortMapping] = [],
         publish_all: bool = False,
+        pull: str = "missing",
         read_only: bool = False,
         restart: Optional[str] = None,
         remove: bool = False,
@@ -1205,6 +1281,7 @@ class ContainerCLI(DockerCLICaller):
         volume_driver: Optional[str] = None,
         volumes_from: List[ValidContainer] = [],
         workdir: Optional[ValidPath] = None,
+        x_passthrough_args: List[str] = [],
     ) -> Union[Container, str, Iterable[Tuple[str, bytes]]]:
         """Runs a container
 
@@ -1280,6 +1357,7 @@ class ContainerCLI(DockerCLICaller):
                 `add_hosts=[("my_host_1", "192.168.30.31"), ("host2", "192.168.80.81")]`
             blkio_weight: Block IO (relative weight), between 10 and 1000,
                 or 0 to disable (default 0)
+            cgroupns: Cgroup namespace mode to use, one of 'host' or 'private'.
             cpu_period: Limit CPU CFS (Completely Fair Scheduler) period
             cpu_quota: Limit CPU CFS (Completely Fair Scheduler) quota
             cpu_rt_period: Limit CPU real-time period in microseconds
@@ -1338,6 +1416,7 @@ class ContainerCLI(DockerCLICaller):
                 the tuple to signify that you want a random free port on the host. For example:
                 `publish=[(80,)]`.
             publish_all: Publish all exposed ports to random ports.
+            pull: Pull image before running ("always"|"missing"|"never") (default "missing").
             read_only: Mount the container's root filesystem as read only.
             restart: Restart policy to apply when a container exits (default "no")
             remove: Automatically remove the container when it exits.
@@ -1361,8 +1440,27 @@ class ContainerCLI(DockerCLICaller):
             The container output as a string if detach is `False` (the default),
             and a `dockertown.Container` if detach is `True`.
         """
+        if not isinstance(command, list):
+            error_message = (
+                "When calling docker.run(), the second argument ('command') "
+                "should be a list. "
+                "Here are some examples:"
+                "docker.run('ubuntu', ['ls']), "
+                "docker.run('ubuntu', ['cat', '/some/file.txt'])"
+            )
+            if isinstance(command, str):
+                # boy this is the most common error in the world
+                error_message += (
+                    f" In your case, command should not be a string. "
+                    f"You can try docker.run('{image}', {command.split()}, ...)."
+                )
+            raise TypeError(error_message)
 
-        image_cli_wrapper.ImageCLI(self.client_config)._pull_if_necessary(image)
+        image_cli = image_cli_wrapper.ImageCLI(self.client_config)
+        if pull == "missing":
+            image_cli._pull_if_necessary(image)
+        elif pull == "always":
+            image_cli.pull(image)
 
         full_cmd = self.docker_cmd + ["container", "run"]
 
@@ -1376,6 +1474,7 @@ class ContainerCLI(DockerCLICaller):
         full_cmd.add_args_list("--cap-drop", cap_drop)
 
         full_cmd.add_simple_arg("--cgroup-parent", cgroup_parent)
+        full_cmd.add_simple_arg("--cgroupns", cgroupns)
         full_cmd.add_simple_arg("--cidfile", cidfile)
 
         full_cmd.add_simple_arg("--cpu-period", cpu_period)
@@ -1473,6 +1572,9 @@ class ContainerCLI(DockerCLICaller):
         self._add_publish_to_command(full_cmd, publish)
         full_cmd.add_flag("--publish-all", publish_all)
 
+        if pull == "never":
+            full_cmd.add_simple_arg("--pull", "never")
+
         full_cmd.add_flag("--read-only", read_only)
         full_cmd.add_simple_arg("--restart", restart)
         full_cmd.add_flag("--rm", remove)
@@ -1504,6 +1606,8 @@ class ContainerCLI(DockerCLICaller):
 
         full_cmd.add_simple_arg("--workdir", workdir)
 
+        full_cmd += x_passthrough_args
+
         full_cmd.append(image)
         full_cmd += command
 
@@ -1810,35 +1914,74 @@ class ContainerStats:
         """Takes a json_dict with container stats from the CLI and
         parses it.
         """
-        self.block_read: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["BlockIO"].split("/")[0]
-        )
-        self.block_write: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["BlockIO"].split("/")[1]
-        )
+        self.block_read: int = parse_byte_size(json_dict["BlockIO"].split("/")[0])
+        self.block_write: int = parse_byte_size(json_dict["BlockIO"].split("/")[1])
         self.cpu_percentage: float = float(json_dict["CPUPerc"][:-1])
         self.container: str = json_dict["Container"]
         self.container_id: str = json_dict["ID"]
         self.memory_percentage: float = float(json_dict["MemPerc"][:-1])
-        self.memory_used: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["MemUsage"].split("/")[0]
-        )
-        self.memory_limit: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["MemUsage"].split("/")[1]
-        )
+        self.memory_used: int = parse_byte_size(json_dict["MemUsage"].split("/")[0])
+        self.memory_limit: int = parse_byte_size(json_dict["MemUsage"].split("/")[1])
         self.container_name: str = json_dict["Name"]
-        self.net_upload: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["NetIO"].split("/")[0]
-        )
-        self.net_download: int = pydantic.parse_obj_as(
-            pydantic.ByteSize, json_dict["NetIO"].split("/")[1]
-        )
+        self.net_upload: int = parse_byte_size(json_dict["NetIO"].split("/")[0])
+        self.net_download: int = parse_byte_size(json_dict["NetIO"].split("/")[1])
 
     def __repr__(self):
         attr = ", ".join(f"{key}={value}" for key, value in self.__dict__.items())
         return f"<{self.__class__} object, attributes are {attr}>"
 
 
+def parse_byte_size(size_str: str) -> int:
+    """Parse a byte size string like '1.5GiB', '512MiB', '2kB' to bytes.
+
+    This replaces the removed pydantic.ByteSize functionality in Pydantic v2.
+    """
+    size_str = size_str.strip()
+
+    # Handle special cases
+    if size_str == "0" or size_str == "0B":
+        return 0
+
+    # Parse the number and unit
+    match = re.match(r"^([\d.]+)\s*([A-Za-z]*)", size_str)
+    if not match:
+        raise ValueError(f"Invalid byte size format: {size_str}")
+
+    number = float(match.group(1))
+    unit = match.group(2).strip()
+
+    # Define multipliers (both SI and IEC units)
+    multipliers = {
+        "": 1,
+        "B": 1,
+        "kB": 1000,
+        "KB": 1000,
+        "k": 1000,
+        "K": 1000,
+        "MB": 1000**2,
+        "M": 1000**2,
+        "GB": 1000**3,
+        "G": 1000**3,
+        "TB": 1000**4,
+        "T": 1000**4,
+        "PB": 1000**5,
+        "P": 1000**5,
+        "KiB": 1024,
+        "Ki": 1024,
+        "MiB": 1024**2,
+        "Mi": 1024**2,
+        "GiB": 1024**3,
+        "Gi": 1024**3,
+        "TiB": 1024**4,
+        "Ti": 1024**4,
+        "PiB": 1024**5,
+        "Pi": 1024**5,
+    }
+
+    multiplier = multipliers.get(unit, 1)
+    return int(number * multiplier)
+
+
 def join_if_not_none(sequence: Optional[list]) -> Optional[str]:
     if sequence is None:
         return None
