@@ -1,10 +1,16 @@
 from __future__ import annotations
 
+import os
+import pty
+import re
+import subprocess
 from datetime import datetime
 from multiprocessing.pool import ThreadPool
 from pathlib import Path
 from subprocess import PIPE, Popen
-from typing import Any, Dict, Iterator, List, Optional, Union, overload
+from typing import Any, Dict, Iterator, List, Optional, Union, overload, Match
+
+import humanfriendly
 
 from ...client_config import ClientConfig, DockerCLICaller, ReloadableObjectFromJson
 from ...components.buildx import cli_wrapper as buildx_cli_wrapper
@@ -17,7 +23,8 @@ from ...utils import (
     stream_stdout_and_stderr,
     to_list,
 )
-from .models import ImageGraphDriver, ImageInspectResult, ImageRootFS
+from .models import ImageGraphDriver, ImageInspectResult, ImageRootFS, ImageHistoryLayer, LayerPullStatus, \
+    LayerPullProgress
 
 
 class Image(ReloadableObjectFromJson):
@@ -36,7 +43,7 @@ class Image(ReloadableObjectFromJson):
         return run(self.docker_cmd + ["image", "inspect", reference])
 
     def _parse_json_object(self, json_object: Dict[str, Any]) -> ImageInspectResult:
-        return ImageInspectResult.parse_obj(json_object)
+        return ImageInspectResult.model_validate(json_object)
 
     def _get_inspect_result(self) -> ImageInspectResult:
         """Only there to allow tools to know the return type"""
@@ -149,18 +156,23 @@ class Image(ReloadableObjectFromJson):
         """
         return ImageCLI(self.client_config).tag(self, new_tag)
 
-    def copy_from(self, path_in_image: ValidPath, destination: ValidPath):
+    def copy_from(
+        self, path_in_image: ValidPath, destination: ValidPath, pull: str = "missing"
+    ):
         """Copy a file from a docker image in the local filesystem.
 
         See the `docker.image.copy_from` command for information about the arguments.
         """
-        return ImageCLI(self.client_config).copy_from(self, path_in_image, destination)
+        return ImageCLI(self.client_config).copy_from(
+            self, path_in_image, destination, pull
+        )
 
     def copy_to(
         self,
         local_path: ValidPath,
         path_in_image: ValidPath,
         new_tag: Optional[str] = None,
+        pull: str = "missing",
     ) -> Image:
         """Copy a file from the local filesystem in a docker image to create a new
         docker image.
@@ -170,7 +182,7 @@ class Image(ReloadableObjectFromJson):
         See the `docker.image.copy_to` command for information about the arguments.
         """
         return ImageCLI(self.client_config).copy_to(
-            self, local_path, path_in_image, new_tag
+            self, local_path, path_in_image, new_tag, pull
         )
 
     def exists(self) -> bool:
@@ -258,12 +270,17 @@ class ImageCLI(DockerCLICaller):
 
         docker_image = ImageCLI(self.client_config)
         full_cmd.append(context_path)
-        image_id = run(full_cmd).strip()
+        image_id = run(full_cmd).splitlines()[-1].strip()
         return docker_image.inspect(image_id)
 
-    def history(self):
+    def history(self, x: str) -> List[ImageHistoryLayer]:
         """Not yet implemented"""
-        raise NotImplementedError
+        full_cmd = self.docker_cmd + ["history", "--no-trunc", "--format", "{{json .}}", x]
+        history: List[ImageHistoryLayer] = [
+            ImageHistoryLayer.parse_json(layer)
+            for layer in run(full_cmd).splitlines()
+        ]
+        return history
 
     def import_(
         self,
@@ -476,6 +493,60 @@ class ImageCLI(DockerCLICaller):
             pool.join()
             return all_images
 
+    def interactive_pull(self, x: str) -> Iterator[LayerPullProgress]:
+        full_cmd = list(map(str, self.docker_cmd)) + ["pull", x]
+
+        master, slave = pty.openpty()
+        p = subprocess.Popen(full_cmd, preexec_fn=os.setsid, stdin=slave, stdout=slave, stderr=slave)
+        os.close(slave)
+
+        ansi_escape = re.compile(rb'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
+        layer_pattern = re.compile(rb"^([0-9a-f]{12}):\s(.*)$")
+        progress_pattern = re.compile(r"^(\d+(\.\d+)?[kMGT]?B)/(\d+(\.\d+)?[kMGT]?B)$")
+
+        buffer: bytes = b""
+
+        while p.poll() is None:
+            try:
+                chunk = os.read(master, 2048)
+            except OSError:
+                return
+            buffer += chunk
+
+            if b"manifest unknown" in buffer:
+                p.wait()
+                raise NoSuchImage(full_cmd, p.returncode, stderr=buffer)
+
+            buffer = ansi_escape.sub(b'', buffer)
+            buffer = buffer.replace(b"\r", b"\n")
+            lines = buffer.splitlines(keepends=True)
+
+            for line in lines[:-1]:
+                line = line.strip()
+                if len(line) > 1:
+                    # process line
+                    match = layer_pattern.match(line)
+                    if match:
+                        layer_id: str = match.group(1).decode("utf-8").strip()
+                        status_str: str = match.group(2).decode("utf-8").strip()
+                        progress: Optional[float] = None
+                        progress_info: Optional[Match] = progress_pattern.match(status_str.split(" ")[-1])
+                        if progress_info:
+                            progress_str: str = status_str.split(" ")[-1]
+                            status_str = status_str[:-len(progress_str)].strip()
+                            partial: float = humanfriendly.parse_size(progress_info.group(1))
+                            total: float = humanfriendly.parse_size(progress_info.group(3))
+                            progress: float = partial / total
+                        status: LayerPullStatus = LayerPullStatus.from_string(status_str)
+
+                        yield LayerPullProgress(
+                            layer_id=layer_id,
+                            status=status,
+                            progress=progress,
+                        )
+
+            buffer = lines[-1]
+
     def _pull_single_tag(self, image_name: str, quiet: bool):
         full_cmd = self.docker_cmd + ["image", "pull"]
 
@@ -657,10 +728,14 @@ class ImageCLI(DockerCLICaller):
             return self.pull(image)
 
     def copy_from(
-        self, image: ValidImage, path_in_image: ValidPath, destination: ValidPath
+        self,
+        image: ValidImage,
+        path_in_image: ValidPath,
+        destination: ValidPath,
+        pull: str = "missing",
     ):
         with container_cli_wrapper.ContainerCLI(self.client_config).create(
-            image
+            image, pull=pull
         ) as tmp_container:
             tmp_container.copy_from(path_in_image, destination)
 
@@ -670,9 +745,10 @@ class ImageCLI(DockerCLICaller):
         local_path: ValidPath,
         path_in_image: ValidPath,
         new_tag: Optional[str] = None,
+        pull: str = "missing",
     ) -> Image:
         with container_cli_wrapper.ContainerCLI(self.client_config).create(
-            base_image
+            base_image, pull=pull
         ) as tmp_container:
             tmp_container.copy_to(local_path, path_in_image)
             return tmp_container.commit(tag=new_tag)
