@@ -9,6 +9,7 @@ from threading import Thread
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, overload
 
 import pydantic
+from pydantic import ConfigDict
 from typing_extensions import Literal
 
 from .exceptions import (
@@ -69,21 +70,22 @@ def to_docker_camel(string):
 
 
 class DockerCamelModel(pydantic.BaseModel):
-    class Config:
-        alias_generator = to_docker_camel
-        allow_population_by_field_name = True
+    model_config = ConfigDict(
+        alias_generator=to_docker_camel,
+        populate_by_name=True,
+    )
 
 
 @overload
 def run(
     args: List[Any],
-    capture_stdout: bool = True,
-    capture_stderr: bool = True,
-    input: bytes = None,
-    return_stderr: Literal[True] = False,
-    env: Dict[str, str] = {},
-    tty: bool = False,
-    retry: int = 3,
+    capture_stdout: bool = ...,
+    capture_stderr: bool = ...,
+    input: bytes = ...,
+    return_stderr: Literal[True] = ...,
+    env: Dict[str, str] = ...,
+    tty: bool = ...,
+    retry: int = ...,
 ) -> Tuple[str, str]:
     ...
 
@@ -91,13 +93,13 @@ def run(
 @overload
 def run(
     args: List[Any],
-    capture_stdout: bool = True,
-    capture_stderr: bool = True,
-    input: bytes = None,
-    return_stderr: Literal[False] = False,
-    env: Dict[str, str] = {},
-    tty: bool = False,
-    retry: int = 3,
+    capture_stdout: bool = ...,
+    capture_stderr: bool = ...,
+    input: bytes = ...,
+    return_stderr: Literal[False] = ...,
+    env: Dict[str, str] = ...,
+    tty: bool = ...,
+    retry: int = ...,
 ) -> str:
     ...
 
@@ -106,7 +108,7 @@ def run(
     args: List[Any],
     capture_stdout: bool = True,
     capture_stderr: bool = True,
-    input: bytes = None,
+    input: Optional[bytes] = None,
     return_stderr: bool = False,
     env: Dict[str, str] = {},
     tty: bool = False,
@@ -186,6 +188,13 @@ def run(
                     completed_process.stdout,
                     completed_process.stderr,
                 )
+            if "manifest unknown" in completed_process.stderr.decode().lower():
+                raise NoSuchImage(
+                    args,
+                    completed_process.returncode,
+                    completed_process.stdout,
+                    completed_process.stderr,
+                )
             if (
                 "Temporary failure in name resolution"
                 in completed_process.stderr.decode().lower()
@@ -306,7 +315,7 @@ def stream_stdout_and_stderr(
         raise DockerException(full_cmd, exit_code, stderr=full_stderr)
 
 
-def format_dict_for_cli(dictionary: Dict[str, str], separator="="):
+def format_dict_for_cli(dictionary: Dict[str, str], separator="=") -> List[str]:
     return [f"{key}{separator}{value}" for key, value in dictionary.items()]
 
 
@@ -337,9 +346,28 @@ def read_env_files(env_files: List[Path]) -> Dict[str, str]:
 
 def all_fields_optional(cls):
     """Decorator function used to modify a pydantic model's fields to all be optional."""
-    for field in cls.__fields__.values():
-        field.required = False
-        field.allow_none = True
+    from typing import get_origin, get_args
+
+    # Make a copy of model_fields to avoid modifying during iteration
+    for field_name, field_info in list(cls.model_fields.items()):
+        # Get the original annotation
+        annotation = field_info.annotation
+
+        # Check if it's already Optional (Union with None)
+        origin = get_origin(annotation)
+        if origin is Union:
+            args = get_args(annotation)
+            if type(None) in args:
+                # Already optional, just set default
+                field_info.default = None
+                continue
+
+        # Make it optional
+        field_info.annotation = Optional[annotation] if annotation else None
+        field_info.default = None
+
+    # Rebuild the model to apply changes
+    cls.model_rebuild(force=True)
     return cls
 
 
@@ -355,3 +383,10 @@ def format_time_for_docker(time_object: Union[datetime, timedelta]) -> str:
         return time_object.strftime("%Y-%m-%dT%H:%M:%S")
     elif isinstance(time_object, timedelta):
         return f"{time_object.total_seconds()}s"
+
+
+def parse_ls_status_count(status_output, status) -> int:
+    try:
+        return int(status_output.split(status + "(")[1].split(")")[0])
+    except IndexError:
+        return 0
